/*
 * Copyright 2013 Yuichiro Moriguchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.morilib.db.relations;

import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import net.morilib.db.engine.SqlEngine;
import net.morilib.db.schema.SqlSchema;

public class RelationIn extends RelationExpression {

	//
	private RelationExpression exp;
	private List<RelationExpression> ins;

	public RelationIn(RelationExpression e,
			List<RelationExpression> in) {
		exp = e;
		ins = new ArrayList<RelationExpression>(in);
	}

	@Override
	public Object eval(SqlEngine v, SqlSchema f,
			RelationTuple tuple, RelationAggregate m,
			List<String> g,
			List<Object> h) throws IOException, SQLException {
		Object o = exp.eval(v, f, tuple, m, g, h);

		for(RelationExpression r : ins) {
			if(o.equals(r.eval(v, f, tuple, m, g, h)))  return TRUE;
		}
		return FALSE;
	}

	public boolean isAggregate() {
		for(RelationExpression e : ins) {
			if(e.isAggregate())  return true;
		}
		return exp.isAggregate();
	}

	@Override
	public Object init(RelationAggregate m) throws SQLException {
		Object o = exp.init(m);

		for(RelationExpression r : ins) {
			if(o.equals(r.init(m)))  return TRUE;
		}
		return FALSE;
	}

}
